@@ -125,6 +125,9 @@ extern "C" {
  * Exported defines
  * ---------------------------------------------------------------------------*/
 
+/* Increment this every time the public ABI changes */
+#define JIM_ABI_VERSION 100
+
 #define JIM_OK 0
 #define JIM_ERR 1
 #define JIM_RETURN 2
@@ -133,7 +136,7 @@ extern "C" {
 #define JIM_SIGNAL 5
 #define JIM_EXIT 6
 /* The following are internal codes and should never been seen/used */
-#define JIM_EVAL 7
+#define JIM_EVAL 7      /* tailcall */
 
 #define JIM_MAX_CALLFRAME_DEPTH 1000 /* default max nesting depth for procs */
 #define JIM_MAX_EVAL_DEPTH 2000 /* default max nesting depth for eval */
@@ -547,6 +550,7 @@ typedef struct Jim_Interp {
                 structure. */
     int local; /* If 'local' is in effect, newly defined procs keep a reference to the old defn */
     int quitting; /* Set to 1 during Jim_FreeInterp() */
+    int safeexpr; /* Set when evaluating a "safe" expression, no var subst or command eval */
     Jim_Obj *liveList; /* Linked list of all the live objects. */
     Jim_Obj *freeList; /* Linked list of all the unused objects. */
     Jim_Obj *currentScriptObj; /* Script currently in execution. */
@@ -566,6 +570,7 @@ typedef struct Jim_Interp {
     Jim_Obj *errorProc; /* Name of last procedure which returned an error */
     Jim_Obj *unknown; /* Unknown command cache */
     Jim_Obj *defer; /* "jim::defer" */
+    Jim_Obj *traceCmdObj; /* If non-null, execution trace command to invoke */
     int unknown_called; /* The unknown command has been invoked */
     int errorFlag; /* Set if an error occurred during execution. */
     void *cmdPrivData; /* Used to pass the private data pointer to
@@ -849,6 +854,8 @@ JIM_EXPORT int Jim_GetBoolean(Jim_Interp *interp, Jim_Obj *objPtr,
 /* integer object */
 JIM_EXPORT int Jim_GetWide (Jim_Interp *interp, Jim_Obj *objPtr,
         jim_wide *widePtr);
+JIM_EXPORT int Jim_GetWideExpr(Jim_Interp *interp, Jim_Obj *objPtr,
+        jim_wide *widePtr);
 JIM_EXPORT int Jim_GetLong (Jim_Interp *interp, Jim_Obj *objPtr,
         long *longPtr);
 #define Jim_NewWideObj  Jim_NewIntObj
@@ -887,13 +894,18 @@ JIM_EXPORT void * Jim_GetAssocData(Jim_Interp *interp, const char *key);
 JIM_EXPORT int Jim_SetAssocData(Jim_Interp *interp, const char *key,
         Jim_InterpDeleteProc *delProc, void *data);
 JIM_EXPORT int Jim_DeleteAssocData(Jim_Interp *interp, const char *key);
+JIM_EXPORT int Jim_CheckAbiVersion(Jim_Interp *interp, int abi_version);
 
 /* Packages C API */
+
 /* jim-package.c */
 JIM_EXPORT int Jim_PackageProvide (Jim_Interp *interp,
         const char *name, const char *ver, int flags);
 JIM_EXPORT int Jim_PackageRequire (Jim_Interp *interp,
         const char *name, int flags);
+#define Jim_PackageProvideCheck(INTERP, NAME) \
+        if (Jim_CheckAbiVersion(INTERP, JIM_ABI_VERSION) == JIM_ERR || Jim_PackageProvide(INTERP, NAME, "1.0", JIM_ERRMSG)) \
+                return JIM_ERR
 
 /* error messages */
 JIM_EXPORT void Jim_MakeErrorMessage (Jim_Interp *interp);
@@ -906,6 +918,8 @@ JIM_EXPORT char *Jim_HistoryGetline(Jim_Interp *interp, const char *prompt);
 JIM_EXPORT void Jim_HistorySetCompletion(Jim_Interp *interp, Jim_Obj *commandObj);
 JIM_EXPORT void Jim_HistoryAdd(const char *line);
 JIM_EXPORT void Jim_HistoryShow(void);
+JIM_EXPORT void Jim_HistorySetMaxLen(int length);
+JIM_EXPORT int Jim_HistoryGetMaxLen(void);
 
 /* Misc */
 JIM_EXPORT int Jim_InitStaticExtensions(Jim_Interp *interp);
